import cv2
import numpy as np

"""
This script summarizes the statiscal analysis performed in:
    'A reliable criterion for the correct delimitation of the foveal avascular zone in diabetic patients',
    by Guisela Fernández-Espinosa, Carlos Ruiz-Tabuenca et al.
    Journal of Personalized Medicine, 2023
Just execute the script, and introduce the nam of a segmented OCTA-A FAZ image.
We provide an example image, 'example_manual_segmentation.tif'
"""


def coord_contour(I):
    # The channel with the largest integer value has the contour information,
    # while the lowest one has the background
    I = cv2.transpose(I)
    # Rotate the image to have the correct orientation
    I = cv2.flip(I, flipCode=1)
    integrals = [sum([sum(i) for i in I[:, :, 0]]), sum([sum(i)
                                                         for i in I[:, :, 1]]), sum([sum(i) for i in I[:, :, 2]])]
    contour = I[:, :, np.argmax(integrals)] - I[:, :, np.argmin(integrals)]
    ret, contour = cv2.threshold(
        contour, 0.5*contour.max(), 255, cv2.THRESH_BINARY)
    coord = np.where(contour == 255)
    return coord


def center(coord):  # Given a set of coordinates, computes the center by averaging all the coordinates
    x0 = np.sum(coord[0]) / coord[0].shape[0]
    y0 = np.sum(coord[1]) / coord[1].shape[0]
    return x0, y0


def cart2pol(coord):  # Given a set of cartesian coordinates, returns the polar form and the center (x0,y0)
    x0, y0 = center(coord)
    x = coord[0] - x0
    y = coord[1] - y0
    # Polar coordinates. Applied as explained in the paper.
    coord_pol = [np.arctan2(y, x), np.sqrt(x * x + y * y)]
    return coord_pol, x0, y0


# Given the FAZ coordinates from an image, calculates acircularity
def calculate_acircularity(coord):
    polar, centerx, centery = cart2pol(coord)
    coordR = coord[1]
    # To do so, the quotient of the SD and the mean of the radial coordinates is computed
    return np.std(coordR)/np.average(coordR)


def pol2cart(coord_pol, x0, y0):  # Reversed algorithm of cart2pol, using x=R*cos(theta) and y=R*sin(theta). A center can also be applied to create an offset
    x = coord_pol[1] * np.cos(coord_pol[0])
    y = coord_pol[1] * np.sin(coord_pol[0])
    coord = [x + x0, y + y0]
    return coord


def sort_coord(coord):  # Given an unordered set of cartesian coordinates, reassembles the points so they are written counter-clockwise (necessary for area calculation)
    coord_pol, x0, y0 = cart2pol(coord)
    coord_pol[0] = coord_pol[0]
    # Once converted to cartesians, we take the angles values and sort the points from minimum to maximum. Thus, the points are ordered in a counter-clockwise order.
    coord_pol[1] = coord_pol[1][coord_pol[0].argsort()]
    coord_pol[0] = coord_pol[0][coord_pol[0].argsort()]
    # Transform back the ordered coordinates to cartesian system.
    coord = pol2cart(coord_pol, x0, y0)
    # Sorted polar and cartesian coordinates are returned.
    return coord_pol, coord


def calculate_area(coord):  # Given a polygon, we apply the Shoelace method to compute the area
    # Arrange the 'coord' coordinates so they are written counter-clockwise.
    polar_coord, coord = sort_coord(coord)
    # Calculate the enclosed area by the Shoelace Method (Also known as Gauss Method, https://en.wikipedia.org/wiki/Shoelace_formula).
    x = coord[0]
    y = coord[1]
    # The sumatory written in the paper can be implemented efficiently using the roll command.
    return 0.5*np.abs(np.dot(x, np.roll(y, 1))-np.dot(y, np.roll(x, 1)))


# All calls to other subalgorithms are packed here.
def main_runtime(filename):
    img = cv2.imread(filename, 1)  # reads the input in color
    contour = coord_contour(img)  # Extract coordinates from image
    acircularity = calculate_acircularity(contour)  # calculate acircularity
    print(f"Acircularity: {acircularity}")
    area = calculate_area(contour)  # Calculate area
    print(f"Area (pixels): {area}")


filename = input(
    "Enter the filename of the segmentated image (i.e. 'example_manual_segmentation.tif'):")
main_runtime(filename)

